@@ -11,7 +11,7 @@
 #   Craig Barratt  <cbarratt@users.sourceforge.net>
 #
 # COPYRIGHT
-#   Copyright (C) 2001  Craig Barratt
+#   Copyright (C) 2001-2003  Craig Barratt
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
 #
 #========================================================================
 #
-# Version 2.0.2, released 6 Oct 2003.
+# Version 2.1.0, released 20 Jun 2004.
 #
 # See http://backuppc.sourceforge.net.
 #
@@ -48,6 +48,7 @@ use File::Compare;
 use Socket;
 use Cwd;
 use Digest::MD5;
+use Config;
 
 sub new
 {
@@ -58,7 +59,7 @@ sub new
         TopDir  => $topDir || '__TOPDIR__',
         BinDir  => $installDir || '__INSTALLDIR__',
         LibDir  => $installDir || '__INSTALLDIR__',
-        Version => '2.0.2',
+        Version => '2.1.0',
         BackupFields => [qw(
                     num type startTime endTime
                     nFiles size nFilesExist sizeExist nFilesNew sizeNew
@@ -70,6 +71,9 @@ sub new
                     num startTime endTime result errorMsg nFiles size
                     tarCreateErrs xferErrs
                 )],
+        ArchiveFields => [qw(
+                    num startTime endTime result errorMsg
+                )],
     }, $class;
     $bpc->{BinDir} .= "/bin";
     $bpc->{LibDir} .= "/lib";
@@ -89,7 +93,7 @@ sub new
     if ( !$noUserCheck
 	    && $bpc->{Conf}{BackupPCUserVerify}
 	    && $> != (my $uid = (getpwnam($bpc->{Conf}{BackupPCUser}))[2]) ) {
-	print("Wrong user: my userid is $>, instead of $uid"
+	print(STDERR "Wrong user: my userid is $>, instead of $uid"
 	    . " ($bpc->{Conf}{BackupPCUser})\n");
 	return;
     }
@@ -128,7 +132,15 @@ sub Lang
 
 sub adminJob
 {
-    return " admin ";
+    my($bpc, $num) = @_;
+    return " admin " if ( !$num );
+    return " admin$num ";
+}
+
+sub isAdminJob
+{
+    my($bpc, $str) = @_;
+    return $str =~ /^ admin/;
 }
 
 sub trashJob
@@ -151,32 +163,33 @@ sub verbose
     return $bpc->{verbose};
 }
 
+sub sigName2num
+{
+    my($bpc, $sig) = @_;
+
+    if ( !defined($bpc->{SigName2Num}) ) {
+	my $i = 0;
+	foreach my $name ( split(' ', $Config{sig_name}) ) {
+	    $bpc->{SigName2Num}{$name} = $i;
+	    $i++;
+	}
+    }
+    return $bpc->{SigName2Num}{$sig};
+}
+
+#
+# Generate an ISO 8601 format timeStamp (but without the "T").
+# See http://www.w3.org/TR/NOTE-datetime and
+# http://www.cl.cam.ac.uk/~mgk25/iso-time.html
+#
 sub timeStamp
 {
     my($bpc, $t, $noPad) = @_;
     my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)
               = localtime($t || time);
-    $year += 1900;
-    $mon++;
-    return "$year/$mon/$mday " . sprintf("%02d:%02d:%02d", $hour, $min, $sec)
-            . ($noPad ? "" : " ");
-}
-
-#
-# An ISO 8601-compliant version of timeStamp.  Needed by the
-# --newer-mtime argument to GNU tar in BackupPC::Xfer::Tar.
-# Also see http://www.w3.org/TR/NOTE-datetime.
-#
-sub timeStampISO
-{
-    my($bpc, $t, $noPad) = @_;
-    my($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)
-              = localtime($t || time);
-    $year += 1900;
-    $mon++;
-    return sprintf("%04d-%02d-%02d ", $year, $mon, $mday)
-         . sprintf("%02d:%02d:%02d", $hour, $min, $sec)
-         . ($noPad ? "" : " ");
+    return sprintf("%04d-%02d-%02d %02d:%02d:%02d",
+		    $year + 1900, $mon + 1, $mday, $hour, $min, $sec)
+	     . ($noPad ? "" : " ");
 }
 
 sub BackupInfoRead
@@ -190,7 +203,7 @@ sub BackupInfoRead
 	binmode(BK_INFO);
         while ( <BK_INFO> ) {
             s/[\n\r]+//;
-            next if ( !/^(\d+\t(incr|full)[\d\t]*$)/ );
+            next if ( !/^(\d+\t(incr|full|partial)[\d\t]*$)/ );
             $_ = $1;
             @{$Backups[@Backups]}{@{$bpc->{BackupFields}}} = split(/\t/);
         }
@@ -207,11 +220,13 @@ sub BackupInfoWrite
     my($i);
 
     flock(LOCK, LOCK_EX) if open(LOCK, "$bpc->{TopDir}/pc/$host/LOCK");
-    unlink("$bpc->{TopDir}/pc/$host/backups.old")
-                if ( -f "$bpc->{TopDir}/pc/$host/backups.old" );
-    rename("$bpc->{TopDir}/pc/$host/backups",
-           "$bpc->{TopDir}/pc/$host/backups.old")
-                if ( -f "$bpc->{TopDir}/pc/$host/backups" );
+    if ( -s "$bpc->{TopDir}/pc/$host/backups" ) {
+	unlink("$bpc->{TopDir}/pc/$host/backups.old")
+		    if ( -f "$bpc->{TopDir}/pc/$host/backups.old" );
+	rename("$bpc->{TopDir}/pc/$host/backups",
+	       "$bpc->{TopDir}/pc/$host/backups.old")
+		    if ( -f "$bpc->{TopDir}/pc/$host/backups" );
+    }
     if ( open(BK_INFO, ">$bpc->{TopDir}/pc/$host/backups") ) {
 	binmode(BK_INFO);
         for ( $i = 0 ; $i < @Backups ; $i++ ) {
@@ -251,11 +266,13 @@ sub RestoreInfoWrite
     my($i);
 
     flock(LOCK, LOCK_EX) if open(LOCK, "$bpc->{TopDir}/pc/$host/LOCK");
-    unlink("$bpc->{TopDir}/pc/$host/restores.old")
-                if ( -f "$bpc->{TopDir}/pc/$host/restores.old" );
-    rename("$bpc->{TopDir}/pc/$host/restores",
-           "$bpc->{TopDir}/pc/$host/restores.old")
-                if ( -f "$bpc->{TopDir}/pc/$host/restores" );
+    if ( -s "$bpc->{TopDir}/pc/$host/restores" ) {
+	unlink("$bpc->{TopDir}/pc/$host/restores.old")
+		    if ( -f "$bpc->{TopDir}/pc/$host/restores.old" );
+	rename("$bpc->{TopDir}/pc/$host/restores",
+	       "$bpc->{TopDir}/pc/$host/restores.old")
+		    if ( -f "$bpc->{TopDir}/pc/$host/restores" );
+    }
     if ( open(RESTORE_INFO, ">$bpc->{TopDir}/pc/$host/restores") ) {
 	binmode(RESTORE_INFO);
         for ( $i = 0 ; $i < @Restores ; $i++ ) {
@@ -268,6 +285,53 @@ sub RestoreInfoWrite
     close(LOCK);
 }
 
+sub ArchiveInfoRead
+{
+    my($bpc, $host) = @_;
+    local(*ARCHIVE_INFO, *LOCK);
+    my(@Archives);
+
+    flock(LOCK, LOCK_EX) if open(LOCK, "$bpc->{TopDir}/pc/$host/LOCK");
+    if ( open(ARCHIVE_INFO, "$bpc->{TopDir}/pc/$host/archives") ) {
+        binmode(ARCHIVE_INFO);
+        while ( <ARCHIVE_INFO> ) {
+            s/[\n\r]+//;
+            next if ( !/^(\d+.*)/ );
+            $_ = $1;
+            @{$Archives[@Archives]}{@{$bpc->{ArchiveFields}}} = split(/\t/);
+        }
+        close(ARCHIVE_INFO);
+    }
+    close(LOCK);
+    return @Archives;
+}
+
+sub ArchiveInfoWrite
+{
+    my($bpc, $host, @Archives) = @_;
+    local(*ARCHIVE_INFO, *LOCK);
+    my($i);
+
+    flock(LOCK, LOCK_EX) if open(LOCK, "$bpc->{TopDir}/pc/$host/LOCK");
+    if ( -s "$bpc->{TopDir}/pc/$host/archives" ) {
+	unlink("$bpc->{TopDir}/pc/$host/archives.old")
+		    if ( -f "$bpc->{TopDir}/pc/$host/archives.old" );
+	rename("$bpc->{TopDir}/pc/$host/archives",
+	       "$bpc->{TopDir}/pc/$host/archives.old")
+		    if ( -f "$bpc->{TopDir}/pc/$host/archives" );
+    }
+    if ( open(ARCHIVE_INFO, ">$bpc->{TopDir}/pc/$host/archives") ) {
+        binmode(ARCHIVE_INFO);
+        for ( $i = 0 ; $i < @Archives ; $i++ ) {
+            my %b = %{$Archives[$i]};
+            printf(ARCHIVE_INFO "%s\n",
+                        join("\t", @b{@{$bpc->{ArchiveFields}}}));
+        }
+        close(ARCHIVE_INFO);
+    }
+    close(LOCK);
+}
+
 sub ConfigRead
 {
     my($bpc, $host) = @_;
@@ -399,7 +463,7 @@ sub RmTreeQuiet
     if ( defined($roots) && length($roots) ) {
       $roots = [$roots] unless ref $roots;
     } else {
-      print "RmTreeQuiet: No root path(s) specified\n";
+      print(STDERR "RmTreeQuiet: No root path(s) specified\n");
     }
     chdir($pwd);
     foreach $root (@{$roots}) {
@@ -412,14 +476,17 @@ sub RmTreeQuiet
 	#
 	if ( !unlink($root) ) {
             if ( -d $root ) {
-                my $d = DirHandle->new($root)
-                  or print "Can't read $pwd/$root: $!";
-                @files = $d->read;
-                $d->close;
-                @files = grep $_!~/^\.{1,2}$/, @files;
-                $bpc->RmTreeQuiet("$pwd/$root", \@files);
-                chdir($pwd);
-                rmdir($root) || rmdir($root);
+                my $d = DirHandle->new($root);
+		if ( !defined($d) ) {
+		    print(STDERR "Can't read $pwd/$root: $!\n");
+		} else {
+		    @files = $d->read;
+		    $d->close;
+		    @files = grep $_!~/^\.{1,2}$/, @files;
+		    $bpc->RmTreeQuiet("$pwd/$root", \@files);
+		    chdir($pwd);
+		    rmdir($root) || rmdir($root);
+		}
             } else {
                 unlink($root) || unlink($root);
             }
@@ -851,8 +918,8 @@ sub NetBiosHostIPFind
     #
     if ( $bpc->{Conf}{NmbLookupFindHostCmd} eq "" ) {
 	print(STDERR "NetBiosHostIPFind: return $host because"
-	           . " \$Conf{NmbLookupFindHostCmd} is empty\n")
-			if ( $bpc->{verbose} );
+	    . " \$Conf{NmbLookupFindHostCmd} is empty\n")
+		if ( $bpc->{verbose} );
 	return $host;
     }
 
@@ -1051,9 +1118,14 @@ sub cmdExecOrEval
     } else {
         $cmd = [split(/\s+/, $cmd)] if ( ref($cmd) ne "ARRAY" );
 	print(STDERR "cmdExecOrEval: about to exec ",
-		      $bpc->execCmd2ShellCmd(@$cmd), "\n")
-			    if ( $bpc->{verbose} );
-        exec(map { m/(.*)/ } @$cmd);		# untaint
+	      $bpc->execCmd2ShellCmd(@$cmd), "\n")
+			if ( $bpc->{verbose} );
+	alarm(0);
+	$cmd = [map { m/(.*)/ } @$cmd];		# untaint
+	#
+	# force list-form of exec(), ie: no shell even for 1 arg
+	#
+        exec { $cmd->[0] } @$cmd;
         print(STDERR "Exec failed for @$cmd\n");
         exit(1);
     }
@@ -1071,9 +1143,9 @@ sub cmdExecOrEval
 #
 # Also, $? should be set when the CHILD pipe is closed.
 #
-sub cmdSystemOrEval
+sub cmdSystemOrEvalLong
 {
-    my($bpc, $cmd, $stdoutCB, @args) = @_;
+    my($bpc, $cmd, $stdoutCB, $ignoreStderr, $pidHandlerCB, @args) = @_;
     my($pid, $out, $allOut);
     local(*CHILD);
     
@@ -1091,7 +1163,7 @@ sub cmdSystemOrEval
     } else {
         $cmd = [split(/\s+/, $cmd)] if ( ref($cmd) ne "ARRAY" );
 	print(STDERR "cmdSystemOrEval: about to system ",
-		      $bpc->execCmd2ShellCmd(@$cmd), "\n")
+	      $bpc->execCmd2ShellCmd(@$cmd), "\n")
 			if ( $bpc->{verbose} );
         if ( !defined($pid = open(CHILD, "-|")) ) {
 	    my $err = "Can't fork to run @$cmd\n";
@@ -1107,11 +1179,26 @@ sub cmdSystemOrEval
 	    # This is the child
 	    #
             close(STDERR);
-	    open(STDERR, ">&STDOUT");
-	    exec(map { m/(.*)/ } @$cmd);		# untaint
-            print("Exec of @$cmd failed\n");
+	    if ( $ignoreStderr ) {
+		open(STDERR, ">", "/dev/null");
+	    } else {
+		open(STDERR, ">&STDOUT");
+	    }
+	    alarm(0);
+	    $cmd = [map { m/(.*)/ } @$cmd];		# untaint
+	    #
+	    # force list-form of exec(), ie: no shell even for 1 arg
+	    #
+	    exec { $cmd->[0] } @$cmd;
+            print(STDERR "Exec of @$cmd failed\n");
             exit(1);
 	}
+
+	#
+	# Notify caller of child's pid
+	#
+	&$pidHandlerCB($pid) if ( ref($pidHandlerCB) eq "CODE" );
+
 	#
 	# The parent gathers the output from the child
 	#
@@ -1129,4 +1216,34 @@ sub cmdSystemOrEval
     return $out;
 }
 
+#
+# The shorter version that sets $ignoreStderr = 0, ie: merges stdout
+# and stderr together.
+#
+sub cmdSystemOrEval
+{
+    my($bpc, $cmd, $stdoutCB, @args) = @_;
+
+    return $bpc->cmdSystemOrEvalLong($cmd, $stdoutCB, 0, undef, @args);
+}
+
+
+#
+# Promotes $conf->{BackupFilesOnly}, $conf->{BackupFilesExclude}
+# to hashes and $conf->{$shareName} to an array
+#
+sub backupFileConfFix
+{
+    my($bpc, $conf, $shareName) = @_;
+
+    $conf->{$shareName} = [ $conf->{$shareName} ]
+                    if ( ref($conf->{$shareName}) ne "ARRAY" );
+    foreach my $param qw(BackupFilesOnly BackupFilesExclude) {
+        next if ( !defined($conf->{$param}) || ref($conf->{$param}) eq "HASH" );
+        $conf->{$param} = [ $conf->{$param} ]
+				if ( ref($conf->{$param}) ne "ARRAY" );
+        $conf->{$param} = { map { $_ => $conf->{$param} }                                                       @{$conf->{$shareName}} };
+    }
+}
+
 1;
